function [img2,nbb2] = IsaacECOL_convert(obj,obj2,img,nbb)
% @IsaacECOL : convertit la matrice img de l'espace couleur obj vers obj2 en modifiant
%					ventuellement le nb de bits ou de plans
%
% (Appeler en TRY-CATCH pour sortie en erreur d'execution)
%
%	- img		: matrice image
%	- nbb		: nombre de bits de la matrice image
%	- obj2	: objet @IsaacECOL
%
% OpenIsaac
%  2018 Alain Clment - Universit d'Angers

%---------------------------------------------------------------------------------------------------

% Copyright 2007 - 2018 Universit d'Angers - Author: Alain Clment
%
% This program is free software; you can redistribute it and/or modify it under the terms of the GNU
% General Public License as published by the Free Software Foundation; either version 3 of the
% License, or any later version. This program is distributed in the hope that it will be useful, but
% WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A
% PARTICULAR PURPOSE. See the GNU General Public License for more details. You should have received
% a copy of the GNU General Public License along with this program; if not, write to the Free
% Software Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.

%---------------------------------------------------------------------------------------------------

                                                                                         %[tomodify]
% conversion LCH (double) supprime car rsultats plan H entre 50 et 500 (bug Matlab 2007a ?)

% profil pour conversion entre ICC et espaces prdfinis
SRGB_ICC = 'sRGB Profile.icc';

if (~isa(obj2,'IsaacECOL') || ~ismember(nbb,obj.bitclass) || ...
		((obj.dimension ~= 0) && (size(img,3) ~= obj.dimension)))
	error('argument incorrect')
end
if ~strcmp(obj.name,'ICC') && strcmp(obj.name,obj2.name)
	img2 = img;
	nbb2 = nbb;
	return
end
if ((obj.dimension ~= 0) && (obj2.dimension ~= 0) && (obj.dimension ~= obj2.dimension))
	error('conversion impossible ou non gre')
end
switch obj.name
	case 'SRGB'
		switch obj2.name
			case 'XYZ'
				[img2,nbb2] = PrepConv(img,nbb,obj2.bitclass);
				clear img
				img2 = applycform(img2,makecform('srgb2xyz'));
			case 'LAB'
				[img2,nbb2] = PrepConv(img,nbb,obj2.bitclass);
				clear img
				if (nbb == 64)
					img2 = im2double(applycform(im2uint16(img2),makecform('srgb2lab')));
					% en double srgb2lab donne des rsultats trs diffrents de [0 1]
				else
					img2 = applycform(img2,makecform('srgb2lab'));
				end	
			case 'HSV'
				[img2,nbb2] = PrepConv(img,nbb,obj2.bitclass);
				clear img
				img2 = ImgConv(rgb2hsv(img2),nbb2);
				% par dfaut le rsultat de rgb2hsv est toujours double
			case 'YIQ'
				[img2,nbb2] = PrepConv(img,nbb,obj2.bitclass);
				clear img
				img2 = ImgConv(rgb2ntsc(img2),nbb2);
				% par dfaut le rsultat de rgb2ntsc est toujours double
			case 'YCbCr'
				[img2,nbb2] = PrepConv(img,nbb,obj2.bitclass);
				clear img
				img2 = rgb2ycbcr(img2);
			case {'XYL','UVL','UpVpL'}
				obj3 = IsaacECOL('XYZ',obj.lightsource);
				[img3,nbb3] = IsaacECOL_convert(obj,obj3,img,nbb);
				clear img
				[img2,nbb2] = IsaacECOL_convert(obj3,obj2,img3,nbb3);
% 			case 'LCH'
% 				obj3 = IsaacECOL('LAB',obj.lightsource);
% 				[img3,nbb3] = IsaacECOL_convert(obj,obj3,img,nbb);
% 				clear img
% 				[img2,nbb2] = IsaacECOL_convert(obj3,obj2,img3,nbb3);
			case 'ICC'
				obj3 = IsaacECOL('ICC',SRGB_ICC);
				% fait juste la mise  jour ventuelle du nb de bita
				[img2,nbb2] = PrepConv(img,nbb,obj3.bitclass);
				clear img
				[img2,nbb2] = IsaacECOL_convert(obj3,obj2,img2,nbb2);
			otherwise
				error('conversion impossible ou non gre')
		end	
	case 'XYZ'
		switch obj2.name
			case 'SRGB'
				[img2,nbb2] = PrepConv(img,nbb,obj2.bitclass);
				clear img
				img2 = applycform(img2,makecform('xyz2srgb'));
			case 'XYL'
				[img2,nbb2] = PrepConv(img,nbb,obj2.bitclass);
				clear img
				img2 = applycform(img2,makecform('xyz2xyl'));
			case 'LAB'
				if isempty(obj.whitepoint)
					C = makecform('xyz2lab');
				else
					C = makecform('xyz2lab','WhitePoint',obj.whitepoint);
				end
				[img2,nbb2] = PrepConv(img,nbb,obj2.bitclass);
				clear img
				if (nbb2 == 64)
					img2 = im2double(applycform(im2uint16(img2),C));
					% en double xyz2lab donne des rsultats trs diffrents de [0 1]
				else
					img2 = applycform(img2,C);
				end	
			case 'UVL'
				[img2,nbb2] = PrepConv(img,nbb,obj2.bitclass);
				clear img
				img2 = applycform(img2,makecform('xyz2uvl'));
			case 'UpVpL'
				[img2,nbb2] = PrepConv(img,nbb,obj2.bitclass);
				clear img
				img2 = applycform(img2,makecform('xyz2upvpl'));
% 			case 'LCH'
% 				obj3 = IsaacECOL('LAB',obj.lightsource);
% 				[img3,nbb3] = IsaacECOL_convert(obj,obj3,img,nbb);
% 				clear img
% 				[img2,nbb2] = IsaacECOL_convert(obj3,obj2,img3,nbb3);
			case {'HSV','YIQ','YCbCr','ICC'}
				obj3 = IsaacECOL('SRGB',obj.lightsource);
				[img3,nbb3] = IsaacECOL_convert(obj,obj3,img,nbb);
				clear img
				[img2,nbb2] = IsaacECOL_convert(obj3,obj2,img3,nbb3);
			otherwise
				error('conversion impossible ou non gre')
		end	
	case 'XYL'
		switch obj2.name
			case 'XYZ'
				[img2,nbb2] = PrepConv(img,nbb,obj2.bitclass);
				clear img
				img2 = applycform(img2,makecform('xyl2xyz'));
			case {'SRGB','LAB','UVL','UpVpL','HSV','YIQ','YCbCr','ICC'} %'LCH'
				obj3 = IsaacECOL('XYZ',obj.lightsource);
				[img3,nbb3] = IsaacECOL_convert(obj,obj3,img,nbb);
				clear img
				[img2,nbb2] = IsaacECOL_convert(obj3,obj2,img3,nbb3);
			otherwise
				error('conversion impossible ou non gre')
		end	
	case 'LAB'
      switch obj2.name
			case 'SRGB'
				[img2,nbb2] = PrepConv(img,nbb,obj2.bitclass);
				clear img
				if (nbb2 == 64)
					img2 = im2double(applycform(im2uint16(img2),makecform('lab2srgb')));
					% en double lab2srgb donne des rsultats trs diffrents de [0 1]
				else
					img2 = applycform(img2,makecform('lab2srgb'));
				end	
			case 'XYZ'
				if isempty(obj.whitepoint)
					C = makecform('lab2xyz');
				else
					C = makecform('lab2xyz','WhitePoint',obj.whitepoint);
				end
				[img2,nbb2] = PrepConv(img,nbb,obj2.bitclass);
				clear img
				if (nbb2 == 64)
					img2 = im2double(applycform(im2uint16(img2),C));
					% en double lab2xyz donne des rsultats trs diffrents de [0 1]
				else
					img2 = applycform(img2,C);
				end	
% 			case 'LCH'
%				[img2,nbb2] = PrepConv(img,nbb,obj2.bitclass);
%				clear img
% 				img2 = applycform(img2,makecform('lab2lch'));
			case {'XYL','UVL','UpVpL'}
				obj3 = IsaacECOL('XYZ',obj.lightsource);
				[img3,nbb3] = IsaacECOL_convert(obj,obj3,img,nbb);
				clear img
				[img2,nbb2] = IsaacECOL_convert(obj3,obj2,img3,nbb3);
			case {'HSV','YIQ','YCbCr','ICC'}
				obj3 = IsaacECOL('SRGB',obj.lightsource);
				[img3,nbb3] = IsaacECOL_convert(obj,obj3,img,nbb);
				clear img
				[img2,nbb2] = IsaacECOL_convert(obj3,obj2,img3,nbb3);
			otherwise
				error('conversion impossible ou non gre')
      end
                                                                                         %[tomodify]
% 	case 'LCH'
% 		switch obj2.name
% 			case 'LAB'
%				[img2,nbb2] = PrepConv(img,nbb,obj2.bitclass);
%				clear img
% 				img2 = applycform(img2,makecform('lch2lab'));
% 			case {'SRGB','XYZ','XYL','UVL','UpVpL','HSV','YIQ','YCbCr','ICC'}
% 				obj3 = IsaacECOL('LAB',obj.lightsource);
% 				[img3,nbb3] = IsaacECOL_convert(obj,obj3,img,nbb);
% 				clear img
% 				[img2,nbb2] = IsaacECOL_convert(obj3,obj2,img3,nbb3);
% 			otherwise
% 				error('conversion impossible ou non gre')
% 		end	
	case 'UVL'
		switch obj2.name
			case 'XYZ'
				[img2,nbb2] = PrepConv(img,nbb,obj2.bitclass);
				clear img
				img2 = applycform(img2,makecform('uvl2xyz'));
			case {'SRGB','XYL','LAB','UpVpL','HSV','YIQ','YCbCr','ICC'} %'LCH'
				obj3 = IsaacECOL('XYZ',obj.lightsource);
				[img3,nbb3] = IsaacECOL_convert(obj,obj3,img,nbb);
				clear img
				[img2,nbb2] = IsaacECOL_convert(obj3,obj2,img3,nbb3);
			otherwise
				error('conversion impossible ou non gre')
		end	
	case 'UpVpL'
		switch obj2.name
			case 'XYZ'
				[img2,nbb2] = PrepConv(img,nbb,obj2.bitclass);
				clear img
				img2 = applycform(img2,makecform('upvpl2xyz'));
			case {'SRGB','XYL','LAB','UVL','HSV','YIQ','YCbCr','ICC'} %'LCH'
				obj3 = IsaacECOL('XYZ',obj.lightsource);
				[img3,nbb3] = IsaacECOL_convert(obj,obj3,img,nbb);
				clear img
				[img2,nbb2] = IsaacECOL_convert(obj3,obj2,img3,nbb3);
			otherwise
				error('conversion impossible ou non gre')
		end	
	case 'HSV'
		switch obj2.name
			case 'SRGB'
				[img2,nbb2] = PrepConv(img,nbb,obj2.bitclass);
				clear img
				img2 = ImgConv(hsv2rgb(im2double(img2)),nbb2);
				% l'entre de hsv2rgb doit tre double
			case {'XYZ','XYL','LAB','UVL','UpVpL','YIQ','YCbCr','ICC'} %'LCH'
				obj3 = IsaacECOL('SRGB',obj.lightsource);
				[img3,nbb3] = IsaacECOL_convert(obj,obj3,img,nbb);
				clear img
				[img2,nbb2] = IsaacECOL_convert(obj3,obj2,img3,nbb3);
			otherwise
				error('conversion impossible ou non gre')
		end	
	case 'YIQ'
		switch obj2.name
			case 'SRGB'
				[img2,nbb2] = PrepConv(img,nbb,obj2.bitclass);
				clear img
				img2 = ImgConv(ntsc2rgb(im2double(img2)),nbb2);
				% l'entre de ntsc2rgb doit tre double
			case {'XYZ','XYL','LAB','UVL','UpVpL','HSV','YCbCr','ICC'} %'LCH'
				obj3 = IsaacECOL('SRGB',obj.lightsource);
				[img3,nbb3] = IsaacECOL_convert(obj,obj3,img,nbb);
				clear img
				[img2,nbb2] = IsaacECOL_convert(obj3,obj2,img3,nbb3);
			otherwise
				error('conversion impossible ou non gre')
		end	
	case 'YCbCr'
		switch obj2.name
			case 'SRGB'
				[img2,nbb2] = PrepConv(img,nbb,obj2.bitclass);
				clear img
				img2 = ycbcr2rgb(img2);
			case {'XYZ','XYL','LAB','UVL','UpVpL','HSV','YIQ','ICC'} %'LCH'
				obj3 = IsaacECOL('SRGB',obj.lightsource);
				[img3,nbb3] = IsaacECOL_convert(obj,obj3,img,nbb);
				clear img
				[img2,nbb2] = IsaacECOL_convert(obj3,obj2,img3,nbb3);
			otherwise
				error('conversion impossible ou non gre')
		end	
	case 'ICC'
		switch obj2.name
			case 'ICC'
				[img2,nbb2] = PrepConv(img,nbb,obj2.bitclass);
				clear img
				try
					% lors de certaines conversions, applycform emet un warning d'arrondi de valeurs non
					% entires vers uint8 ou uint16
					warning off
					img2 = applycform(img2,makecform('icc',obj.iccprofile,obj2.iccprofile));
					warning on
				catch
					error('conversion impossible ou non gre')
				end	
			case 'SRGB'
				obj3 = IsaacECOL('ICC',SRGB_ICC);
				[img3,nbb3] = IsaacECOL_convert(obj,obj3,img,nbb);
				clear img
				% fait juste la mise  jour ventuelle du nb de bita
				[img2,nbb2] = PrepConv(img3,nbb3,obj2.bitclass);
			case {'XYZ','XYL','LAB','UVL','UpVpL','HSV','YIQ','YCbCr'} %'LCH'
				obj3 = IsaacECOL('SRGB',obj2.lightsource);
				[img3,nbb3] = IsaacECOL_convert(obj,obj3,img,nbb);
				clear img
				[img2,nbb2] = IsaacECOL_convert(obj3,obj2,img3,nbb3);
			otherwise
				error('conversion impossible ou non gre')
		end	
	otherwise
		error('conversion impossible ou non gre')
end

%---------------------------------------------------------------------------------------------------

function [img2,nbb2] = PrepConv(img,nbb,bitclass)

minb = min(bitclass);
maxb = max(bitclass);
if (nbb < minb)
	nbb2 = minb;
	img2 = ImgConv(img,nbb2);
elseif (nbb > maxb)
	nbb2 = maxb;
	img2 = ImgConv(img,nbb2);
else
	nbb2 = nbb;
	img2 = img;
end	

%---------------------------------------------------------------------------------------------------

function img2 = ImgConv(img,nbb)
	switch nbb
		case 8
			img2 = im2uint8(img);
		case 16
			img2 = im2uint16(img);
		case 32
			img2 = im2single(img);
			% clip des ventuelles valeurs < 0 ou > 1 conserves par im2single si img single/double
			% (cas de la conversion ntsc qui retourne des valeurs ngatives)
			n = size(img,3);
			for i = 1:n
				img2(:,:,i) = imadjust(img2(:,:,i),[],[]);
			end	
		case 64
			img2 = im2double(img);
			% clip des ventuelles valeurs < 0 ou > 1 conserves par im2double si img single/double
			% (cas de la conversion ntsc qui retourne des valeurs ngatives)
			n = size(img,3);
			for i = 1:n
				img2(:,:,i) = imadjust(img2(:,:,i),[],[]);
			end	
		otherwise
			error('conversion impossible ou non gre')
	end
