function obj = IsaacHISTO(varargin)
% @IsaacHISTO : constructeur
%
% (Appeler en TRY-CATCH pour sortie en erreur d'execution)
%
%	obj = IsaacHISTO(objIMG,useroi,plans)
%		=> objet @IsaacHISTO calcul  partir de objIMG
%
%	obj = IsaacHISTO(filename)
%		=> objet @IsaacHISTO lu dans un fichier
%
%	obj = IsaacHISTO()
%		=> objet @IsaacHISTO par dfaut
%			(non utilis mais obligatoire pour load si aucun objet cr prcdement)
%
%	- useroi		: true si limitation  la ROI de objIMG
%	- plans		: matrice(1,p) des numros de plans slectionns dans objIMG, p <= nb plans objIMG
%						=> impose le nombre et l'ordre des composantes
%  - filename	: nom du fichier dans rpertoire courant ou nom complet
%
% OpenIsaac
%  2018 Alain Clment - Universit d'Angers

%---------------------------------------------------------------------------------------------------

% Copyright 2007 - 2018 Universit d'Angers - Author: Alain Clment
%
% This program is free software; you can redistribute it and/or modify it under the terms of the GNU
% General Public License as published by the Free Software Foundation; either version 3 of the
% License, or any later version. This program is distributed in the hope that it will be useful, but
% WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A
% PARTICULAR PURPOSE. See the GNU General Public License for more details. You should have received
% a copy of the GNU General Public License along with this program; if not, write to the Free
% Software Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.

%---------------------------------------------------------------------------------------------------

% IMAGE D'ORIGINE
%
%	ima_name		: nom de l'image
%	ima_nbpix	: nombre de pixels de l'image
%	ima_dimz		: nombre de plans de l'image
%	ima_nbb		: nombre de bits de l'image
%	ima_useroi  : true si limitation  la ROI de l'image
%
% HISTOGRAMME COMPACT
%
% 	histo			: matrice(n,p) des p-uplets de l'histogramme compact, 1 <= pi <= ima_dimz,
%						dans l'ordre et en nombre de la matrice plans
% 	counts		: matrice(n,1) des effectifs de l'histogramme compact
%	bitclass		: nombre de bits (1/8/16/32/64) de codage de la matrice histo
%						= classe de bits de objIMG
% 	plans			: matrice(1,p) des numros de plans slectionns dans objIMG, 1 <= p <= ima_dimz
%
%	valnb			: nombre de valeurs de l'histogramme compact
%	valdim		: nombre de plans de l'histogramme compact (= p)
%	valmin		: matrice(1,p) des valeurs min de la matrice histo
%	valmax		: matrice(1,p) des valeurs max de la matrice histo
%	valsup		: limite suprieure de valmax en fonction de bitclass
%	effmin		: min de la matrice counts
%	effmax		: max de la matrice counts
%	effsum		: somme des effectifs de la matrice counts
%
%  ATTENTION : counts cod en DOUBLE et histo cod dans la classe de bits de l'image (1/8/16/32/64) 
%
%	version		: version de l'objet 

%---------------------------------------------------------------------------------------------------

objversion = 1;

%---------------------------------------------------------------------------------------------------

global ISAACPREF

switch nargin
	case 3
		% Objet calcul
		% image d'origine
		objIMG = varargin{1};
		if ~isa(objIMG,'IsaacIMG')
			error('argument incorrect')
		end
		obj.ima_name = IsaacIMG_get(objIMG,'ImgName');
		obj.ima_nbpix = IsaacIMG_get(objIMG,'DimX') * IsaacIMG_get(objIMG,'DimY');
		obj.ima_dimz = IsaacIMG_get(objIMG,'DimZ');
		obj.ima_nbb = IsaacIMG_get(objIMG,'NbBits');
		obj.bitclass = IsaacIMG_get(objIMG,'BitClass');
		if (obj.bitclass < 32)
			obj.valsup = 2^obj.bitclass - 1;
		else
			obj.valsup = 1;
		end	
		img = IsaacIMG_get(objIMG,'Img');
		% limitation ventuelle de l'image  la ROI
		obj.ima_useroi = varargin{2};
		if ~islogical(obj.ima_useroi)
			error('argument incorrect')
		end
		if obj.ima_useroi
			mask = IsaacIMG_get(objIMG,'RoiMask');
			masknb = sum(sum(mask));
			if masknb
				img2 = reshape(img(repmat(mask,[1,1,obj.ima_dimz])),1,masknb,obj.ima_dimz);
			else
				img2 = [];
			end
		else
			img2 = img;
		end
		clear img
		% ordonnancement des plans de l'image
		obj.plans = varargin{3};
		if ((size(obj.plans,1) ~= 1) || (size(obj.plans,2) > obj.ima_dimz) || ...
			any(obj.plans < 1) || any(obj.plans > obj.ima_dimz) || ...
			(length(unique(obj.plans)) ~= length(obj.plans))) % erreur si doublons
			error('argument incorrect')
		end
		if ~isempty(img2)
			obj.valdim = size(obj.plans,2);
			img3 = img2; % pour conserver le type
			for i = 1:obj.valdim
				img3(:,:,i) = img2(:,:,obj.plans(i));
			end
			if (obj.valdim < obj.ima_dimz)
				img3(:,:,obj.valdim+1:obj.ima_dimz) = [];
			end	
		else 
			obj.valdim = 0;
			obj.plans = [];
			img3 = [];
		end
		clear img2
		% calcul de l'histogramme
		if isempty(img3)
			obj.histo = [];
			obj.counts = [];
			obj.valnb = 0;
			obj.valmin = [];
			obj.valmax = [];
			obj.effmin = 0;
			obj.effmax = 0;
			obj.effsum = 0;
		else	
			if (((obj.valdim <= 3)) && (obj.bitclass <=8))
				% utilisation de imhist3
				if (obj.bitclass == 1)
					img4 = uint8(img3); % (conserve le 1 en 1 contrairement  im2uint8)
				else
					img4 = img3;
				end
				clear img3
				if (obj.valdim < 3)
					img4(:,:,obj.valdim+1:3) = repmat(img4(:,:,1),[1,1,3-obj.valdim]);
					% les plans rajouts ne seront pas utiliss
					ind = false(1,3);
					ind(1:obj.valdim) = true;
               [histo,obj.counts] = IsaacHist3(img4,uint8([0,0,0;255,255,255;ind]));
            else
               [histo,obj.counts] = IsaacHist3(img4,uint8([0,0,0;255,255,255;1,1,1]));
            end
				if (obj.bitclass == 1)
					obj.histo = logical(histo);
				else
					obj.histo = histo;
				end	
				clear histo
			else
				% utilisation de histn
				if (obj.bitclass ~= 64)
					img4 = double(img3); % (conserve les valeurs contrairement  im2double)
				else
					img4 = img3;
				end
				clear img3
            [histo,obj.counts] = IsaacHistN(img4);
				if (obj.bitclass ~= 64)
					switch obj.bitclass
						case 1
							obj.histo = logical(histo);
						case 8
							obj.histo = uint8(histo);
						case 16
							obj.histo = uint16(histo);
						case 32
							obj.histo = single(histo);
					end
				else
					obj.histo = histo;
				end	
				clear histo
			end
			obj.valnb = size(obj.counts,1);
			obj.valmin = obj.histo(1,:);
			obj.valmax = obj.histo(obj.valnb,:);
			obj.effmin = min(obj.counts);
			obj.effmax = max(obj.counts);
			obj.effsum = sum(obj.counts);
		end
		obj.version = objversion;
		obj = class(orderfields(obj),'IsaacHISTO'); % orderfields car class est sensible  l'ordre	
	case 1
		% Lecture dans un fichier
		try
			warning off	
			var = load('-mat',varargin{1},'HISTO'); % variable HISTO dans le fichier
			warning on
			obj = var.HISTO;
			if ~isa(obj,'IsaacHISTO')
				% erreur affiche par loadobj
				error('pour provoquer le catch')
			end
		catch
			error('fichier invalide')
		end
		if (obj.valdim > ISAACPREF.ImMaxPlans)
			error('le nombre de plans dpasse la limite fixe en prfrences')
		end
	case 0
		% Objet par dfaut non utilis mais obligatoire
		obj = struct('ima_name',{},'ima_nbpix',{},'ima_dimz',{},'ima_nbb',{},'ima_useroi',{}, ...
			'histo',{},'counts',{},'bitclass',{},'plans',{},'valnb',{},'valdim',{},'valmin',{}, ...
			'valmax',{},'valsup',{},'effmin',{},'effmax',{},'effsum',{},'version',{});
		obj = class(orderfields(obj),'IsaacHISTO'); % orderfields car class est sensible  l'ordre
	otherwise
		error('nombre d''arguments incorrect')
end