function val = IsaacIMG_compare(obj1,obj2,propname)
% @IsaacIMG : retourne true si propname identique pour obj1 et obj2, false sinon
%
% propname			comparaison
% --------			-----------
%	'Bits'			bitclass && nbb
%	'ColorSpace'	iscolorspace && colorspace
%	'DimXY'			dimx && dimy
%	'DimXYZ'       dimx && dimy && dimz
%	'DimZ'			dimz
%	'Img'          bitclass && dimx && dimy && dimz && img
%	'ImgName'		imgname
%	'Roi'          isroi && roi		
%	'RoiSpa'       IsaacROI_get('IsRoiSpa' && 'RoiSpa' && 'ImaRef' && 'RoiSpaIn')
%	'RoiVal'       IsaacROI_get('IsRoiVal' && 'RoiVal' && 'ImaRef' && 'RoiValIn')
%
%    - obj1 : objet @IsaacIMG
%    - obj2 : objet @IsaacIMG

% OpenIsaac
%  2018 Alain Clment - Universit d'Angers

%---------------------------------------------------------------------------------------------------

% Copyright 2007 - 2018 Universit d'Angers - Author: Alain Clment
%
% This program is free software; you can redistribute it and/or modify it under the terms of the GNU
% General Public License as published by the Free Software Foundation; either version 3 of the
% License, or any later version. This program is distributed in the hope that it will be useful, but
% WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A
% PARTICULAR PURPOSE. See the GNU General Public License for more details. You should have received
% a copy of the GNU General Public License along with this program; if not, write to the Free
% Software Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.

%---------------------------------------------------------------------------------------------------

if ~isa(obj1,'IsaacIMG') || ~isa(obj2,'IsaacIMG')
	error('arguments incorrects')
end
switch propname
	case 'Bits'
		val = (obj1.bitclass == obj2.bitclass) && (obj1.nbb == obj2.nbb);
	case 'ColorSpace'
		val = (obj1.iscolorspace == obj2.iscolorspace) &&  ...
			IsaacECOL_isequal(obj1.colorspace,obj2.colorspace);
	case 'DimXY'
		val = (obj1.dimx == obj2.dimx) && (obj1.dimy == obj2.dimy);
	case 'DimXYZ'
		val = (obj1.dimx == obj2.dimx) && (obj1.dimy == obj2.dimy) && (obj1.dimz == obj2.dimz);	
	case 'DimZ'
		val = (obj1.dimz == obj2.dimz);
	case 'Img'
		val = (obj1.bitclass == obj2.bitclass) && (obj1.dimx == obj2.dimx) &&  ...
			  (obj1.dimy == obj2.dimy) && (obj1.dimz == obj2.dimz) && isequal(obj1.img,obj2.img);
	case 'ImgName'
		val = strcmp(obj1.imgname,obj2.imgname);
	case 'Roi'
		val = (obj1.isroi == obj2.isroi) && IsaacROI_isequal(obj1.roi,obj2.roi);
	case 'RoiSpa'
		val = (IsaacROI_get(obj1.roi,'IsRoiSpa') == IsaacROI_get(obj2.roi,'IsRoiSpa')) && ...
			  (IsaacROI_get(obj1.roi,'RoiSpa') == IsaacROI_get(obj2.roi,'RoiSpa')) && ...
			  (IsaacROI_get(obj1.roi,'ImaRef') == IsaacROI_get(obj2.roi,'ImaRef')) && ...
			  (IsaacROI_get(obj1.roi,'RoiSpaIn') == IsaacROI_get(obj2.roi,'RoiSpaIn'));
	case 'RoiVal'
		val = (IsaacROI_get(obj1.roi,'IsRoiVal') == IsaacROI_get(obj2.roi,'IsRoiVal')) && ...
			  (IsaacROI_get(obj1.roi,'RoiVal') == IsaacROI_get(obj2.roi,'RoiVal')) && ...
			  (IsaacROI_get(obj1.roi,'ImaRef') == IsaacROI_get(obj2.roi,'ImaRef')) && ...
			  (IsaacROI_get(obj1.roi,'RoiValIn') == IsaacROI_get(obj2.roi,'RoiValIn'));
	otherwise
		error('argument incorrect')
end
