function ud = HistoEdit_maj(fighnd,ud,varargin)
% Mise  jour de l'affichage dans HistoEdit
% (partag par HistoEdit_cb et HistoEdit_upd)
%
%	ud = HistoEdit_maj(fighnd,ud,maj1,...,majN)
%
% - fighnd	: handle figure HistoEdit
% - ud		: champ UserData de HistoEdit
%
%	propname			
%	--------
%	'Effectifs'	: maj des champs d'effectif
%	'Labels'		: maj graduations axes
%	'Nom'			: maj nom histo
%	'Plans'		: maj popups plans
%	'Vue3D'		: maj boutons vue 3D
%
% Tool for OpenIsaac
%  2018 Alain Clment - Universit d'Angers

%---------------------------------------------------------------------------------------------------

% Copyright 2007 - 2018 Universit d'Angers - Author: Alain Clment
%
% This program is free software; you can redistribute it and/or modify it under the terms of the GNU
% General Public License as published by the Free Software Foundation; either version 3 of the
% License, or any later version. This program is distributed in the hope that it will be useful, but
% WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A
% PARTICULAR PURPOSE. See the GNU General Public License for more details. You should have received
% a copy of the GNU General Public License along with this program; if not, write to the Free
% Software Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.

%---------------------------------------------------------------------------------------------------

if (nargin < 3)
	error('nombre d''arguments incorrect')
end
propargin = varargin;
while ~isempty(propargin)
	propname = propargin{1};
	propargin = propargin(2:end);
	switch propname
		case 'Effectifs'
			UpdateEff
		case 'Labels'
			UpdateLabels
		case 'Nom'
			UpdateName
		case 'Plans'
			UpdatePlans
		case 'Vue3D'	
			UpdateVue3D
		otherwise
			error('argument incorrect')
	end
end
			
% --- Nested fonction ------------------------------------------------------------------------------

function UpdateName
	
historef = IsaacHISTO_get(ud.Param,'HistoRef');
s = ['HISTO (',num2str(historef.nb),' x ',num2str(historef.dim),' - ',num2str(historef.bitclass)];
if (historef.bitclass == 1)
	set(findobj(fighnd,'Tag','StxtParamInfo'),'String',[s, ' bit)'])
else
	set(findobj(fighnd,'Tag','StxtParamInfo'),'String',[s, ' bits)'])
end	

end % fin nested fonction

% --- Nested fonction ------------------------------------------------------------------------------

function UpdatePlans
	
if ud.AffDim > 3
	set(findobj(fighnd,'Tag','PopupPlan1'),'Enable','off','String',' ','Value',1)
	set(findobj(fighnd,'Tag','PopupPlan2'),'Enable','off','String',' ','Value',1)
	set(findobj(fighnd,'Tag','PopupPlan3'),'Enable','off','String',' ','Value',1)
	ud.ShowGraph = false;
	set(findobj(fighnd,'Tag','RadShowGraph'),'Enable','off','Value',0)
	set(findobj(fighnd,'Tag','RadShowInfos'),'Value',1)
else
	switch ud.AffDim
		case 1
			set(findobj(fighnd,'Tag','PopupPlan1'),'Enable','on','String',ud.PlansStr, ...
				'Value',ud.AffPlans(1))
			set(findobj(fighnd,'Tag','PopupPlan2'),'Enable','off','String',' ','Value',1)
			set(findobj(fighnd,'Tag','PopupPlan3'),'Enable','off','String',' ','Value',1)
		case 2
			set(findobj(fighnd,'Tag','PopupPlan1'),'Enable','on','String',ud.PlansStr, ...
				'Value',ud.AffPlans(1))
			set(findobj(fighnd,'Tag','PopupPlan2'),'Enable','on','String',ud.PlansStr, ...
				'Value',ud.AffPlans(2))
			set(findobj(fighnd,'Tag','PopupPlan3'),'Enable','off','String',' ','Value',1)
		case 3
			set(findobj(fighnd,'Tag','PopupPlan1'),'Enable','on','String',ud.PlansStr, ...
				'Value',ud.AffPlans(1))
			set(findobj(fighnd,'Tag','PopupPlan2'),'Enable','on','String',ud.PlansStr, ...
				'Value',ud.AffPlans(2))
			set(findobj(fighnd,'Tag','PopupPlan3'),'Enable','on','String',ud.PlansStr, ...
				'Value',ud.AffPlans(3))
	end
	set(findobj(fighnd,'Tag','RadShowGraph'),'Enable','on','Value',ud.ShowGraph)
	set(findobj(fighnd,'Tag','ChkShowInfos'),'Value',~ud.ShowGraph)
end

end % fin nested fonction

% --- Nested fonction ------------------------------------------------------------------------------

function UpdateEff
	
countsref = IsaacHISTO_get(ud.Param,'CountsRef');
% effectifs
set(findobj(fighnd,'Tag','StxtEffMin'),'String',num2str(countsref.effmin))
set(findobj(fighnd,'Tag','StxtEffMax'),'String',num2str(countsref.effmax))
set(findobj(fighnd,'Tag','StxtEffTotal'),'String',num2str(countsref.effsum))
if (ud.ShowGraph)
	set(findobj(fighnd,'Tag','EtxtEffVisMin'),'Enable','on','String',num2str(ud.VisMin))
	set(findobj(fighnd,'Tag','EtxtEffVisMax'),'Enable','on','String',num2str(ud.VisMax))
	% colorbar
	if (ud.AffDim >= 2)
		vraimin = max(ud.VisMin,countsref.effmin);
		vraimax = max(vraimin,ud.VisMax);
		plage = 1 + vraimax - vraimin;
		if plage > 6
			ud.NbColor = 6;
			if plage < 12
				set(ud.AxColorbar, 'Xtick', [1 2 3 4 5 5.7])
				ud.TabSeuil = [vraimin,vraimin+1,vraimin+2,vraimin+3,vraimin+4,vraimin+5,vraimax+1];
			else
				set(ud.AxColorbar, 'Xtick', [0.7 1.7 2.7 3.7 4.7 5.7])
				ud.TabSeuil = [vraimin,round(vraimin+plage/6),round(vraimin+plage/3), ...
				round(vraimin+plage/2),round(vraimin+2*plage/3),round(vraimin+5*plage/6),vraimax+1];
			end	
		else	
			ud.NbColor = plage;
			set(ud.AxColorbar, 'Xtick',1:ud.NbColor)
			ud.TabSeuil = [vraimin:vraimax,vraimax+1];
		end	
		set(ud.AxColorbar,'Visible','on','XtickLabel',ud.TabSeuil(1:ud.NbColor), ...
			'XLim',[0.5,ud.NbColor+0.5])
		set(get(ud.AxColorbar,'Children'),'Visible','on', ...
			'Cdata',uint8(1:ud.NbColor),'Xdata',[1,ud.NbColor])
		map = gray(ud.NbColor+6);			
		ud.TabColor = map(4:ud.NbColor+3,:);
	else
		set(ud.AxColorbar,'Visible','off')
		set(get(ud.AxColorbar,'Children'),'Visible','off')
	end	
else
	set(findobj(fighnd,'Tag','EtxtEffVisMin'),'Enable','off','String','')
	set(findobj(fighnd,'Tag','EtxtEffVisMax'),'Enable','off','String','')
	set(ud.AxColorbar,'Visible','off')
	set(get(ud.AxColorbar,'Children'),'Visible','off')
end

end % fin nested fonction

% --- Nested fonction ------------------------------------------------------------------------------

function UpdateVue3D
	
if (ud.ShowGraph && (ud.AffDim >= 2))
	set(findobj(fighnd,'Tag','ButHistG'),'Enable','on')
	if (ud.EL > 0) 
		set(findobj(fighnd,'Tag','ButHistB'),'Enable','on')
	end	
	set(findobj(fighnd,'Tag','ButHistC'),'Enable','on')
	if (ud.EL < 90) 
		set(findobj(fighnd,'Tag','ButHistH'),'Enable','on')
	end	
	set(findobj(fighnd,'Tag','ButHistD'),'Enable','on')
else
	set(findobj(fighnd,'Tag','ButHistG'),'Enable','off')
	set(findobj(fighnd,'Tag','ButHistB'),'Enable','off')
	set(findobj(fighnd,'Tag','ButHistC'),'Enable','off')
	set(findobj(fighnd,'Tag','ButHistH'),'Enable','off')
	set(findobj(fighnd,'Tag','ButHistD'),'Enable','off')
end

end % fin nested fonction

% --- Nested fonction ------------------------------------------------------------------------------

function UpdateLabels

historef = IsaacHISTO_get(ud.Param,'HistoRef');
switch historef.bitclass
	case 1
		ud.AxTick1 = [0 1];
		ud.AxTick2 = [0 1];
		ud.AxXmin = -0.5;
		ud.AxXmax = 1.5;
		ud.FormatStr = '%2i';
	case 8
		ud.AxTick1 = [0 31 63 95 127 159 191 223 255];
		ud.AxTick2 = [0 63 127 191 255];
		ud.AxXmin = -5;
		ud.AxXmax = 260;
		ud.FormatStr = '%4i';
	case 16
		ud.AxTick1 = [0 16191 32639 49087 65535];
		ud.AxTick2 = [0 16191 32639 49087 65535];
		ud.AxXmin = -1500;
		ud.AxXmax = 67000;
		ud.FormatStr = '%6i';
	case 32
		ud.AxTick1 = [0 0.12 0.25 0.37 0.50 0.62 0.75 0.87 1];
		ud.AxTick2 = [0 0.25 0.50 0.75 1];
		ud.AxXmin = -0.02;
		ud.AxXmax = 1.02;
		ud.FormatStr = '%7.4f';
	case 64
		ud.AxTick1 = [0 0.12 0.25 0.37 0.50 0.62 0.75 0.87 1];
		ud.AxTick2 = [0 0.25 0.50 0.75 1];
		ud.AxXmin = -0.02;
		ud.AxXmax = 1.02;
		ud.FormatStr = '%7.4f';
end

end % fin nested fonction

%---------------------------------------------------------------------------------------------------

end % fin HistoEdit_maj en raison des nested fonctions
