function ImageAjust_cb(choix)
% Callback ImageAjust
%
% Tool for OpenIsaac
%  2018 Alain Clment - Universit d'Angers

%---------------------------------------------------------------------------------------------------

% Copyright 2007 - 2018 Universit d'Angers - Author: Alain Clment
%
% This program is free software; you can redistribute it and/or modify it under the terms of the GNU
% General Public License as published by the Free Software Foundation; either version 3 of the
% License, or any later version. This program is distributed in the hope that it will be useful, but
% WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A
% PARTICULAR PURPOSE. See the GNU General Public License for more details. You should have received
% a copy of the GNU General Public License along with this program; if not, write to the Free
% Software Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.

%---------------------------------------------------------------------------------------------------

% Utilise la rception d'vnements causs par (auteur = soi-mme) pour mise  jour graphique

%---------------------------------------------------------------------------------------------------

global ISAACINFO

switch(choix)
case 'CloseBox'
	ud = get(gcbf,'UserData');
	if ~ISAACINFO.IsEnd
		IsaacEventUpdListe(gcbf,[])
	end
	rmpath(ud.ToolDir)
	closereq
case 'ButResize-'
	cible = IsaacImageCible;
	objIMG = IsaacImage_get(cible,'IMG');
	try
		IsaacImage_set(cible,gcbf,'IMG',IsaacIMG_resize(objIMG,0.5));
	catch exception
		IsaacMessageErreur(IsaacError(exception))
	end
	% maj graphique par ImageAjust_upd	
case 'ButResize+'
	cible = IsaacImageCible;
	objIMG = IsaacImage_get(cible,'IMG');
	try
		IsaacImage_set(cible,gcbf,'IMG',IsaacIMG_resize(objIMG,2));
	catch exception
		IsaacMessageErreur(IsaacError(exception))
	end
	% maj graphique par ImageAjust_upd
case 'ButCrop'
	cible = IsaacImageCible;
	rect = IsaacImage_mouserect(cible);
	if isempty(rect)
		return
	end
	objIMG = IsaacImage_get(cible,'IMG');
	try
		IsaacImage_set(cible,gcbf,'IMG',IsaacIMG_crop(objIMG,rect));
	catch exception
		IsaacMessageErreur(IsaacError(exception))
	end
	% maj graphique par ImageAjust_upd
case 'ButPlanDown'
	val = get(findobj(gcbf,'Tag','ListPlans'),'Value'); % pas de slection multiple possible
	if isempty(val) || (val == 1)
		return
	end
	cible = IsaacImageCible;
	objIMG = IsaacImage_get(cible,'IMG');
	plans = 1:IsaacIMG_get(objIMG,'DimZ');
	plans(val) = val-1;
	plans(val-1) = val;
	ud = get(gcbf,'UserData');
	ud.Plan = val-1;
	set(gcbf,'UserData',ud)
	try
		IsaacImage_set(cible,gcbf,'IMG',IsaacIMG_reorder(objIMG,plans));
	catch exception
		IsaacMessageErreur(IsaacError(exception))
	end
	% maj graphique par ImageAjust_upd
case 'ButPlanUp'
	val = get(findobj(gcbf,'Tag','ListPlans'),'Value'); % pas de slection multiple possible
	cible = IsaacImageCible;
	paramr = IsaacImage_get(cible,'ParamR');
	if isempty(val) || (val == paramr.dimz)
		return
	end
	objIMG = IsaacImage_get(cible,'IMG');
	plans = 1:paramr.dimz;
	plans(val) = val+1;
	plans(val+1) = val;
	ud = get(gcbf,'UserData');
	ud.Plan = val+1;
	set(gcbf,'UserData',ud)
	try
		IsaacImage_set(cible,gcbf,'IMG',IsaacIMG_reorder(objIMG,plans));
	catch exception
		IsaacMessageErreur(IsaacError(exception))
	end
	% maj graphique par ImageAjust_upd
case 'ButRemovePlan'
	val = get(findobj(gcbf,'Tag','ListPlans'),'Value'); % pas de slection multiple possible
	if isempty(val)
		return
	end	
	cible = IsaacImageCible;
	objIMG = IsaacImage_get(cible,'IMG');
	try
		IsaacImage_set(cible,gcbf,'IMG',IsaacIMG_subplans(objIMG,val));
	catch exception
		IsaacMessageErreur(IsaacError(exception))
	end
	% maj graphique par ImageAjust_upd
case 'PopupNbBits'
	val = get(gcbo,'Value');
	if (val <= 16)
		nbbits = val;
	elseif (val == 17)
		nbbits = 32;
	else
		nbbits = 64;
	end	
	cible = IsaacImageCible;
	paramr = IsaacImage_get(cible,'ParamR');
	if (paramr.nbbits ~= nbbits)
		objIMG = IsaacImage_get(cible,'IMG');
		IsaacImage_set(cible,gcbf,'IMG',IsaacIMG_changebits(objIMG,nbbits));
	end
	% maj graphique par ImageAjust_upd
case 'ButChangeDyn'
	% ajustement dynamique vectoriel (mmes min et max pour tous les plans)
	cible = IsaacImageCible;
	ud = get(gcbf,'UserData');
	try
		IsaacImage_set(cible,gcbf,'IMG', ...
			IsaacIMG_changedyn(IsaacImage_get(cible,'IMG'),ud.DynMin,ud.DynMax));
	catch exception
		IsaacMessageErreur(IsaacError(exception))
	end
	% maj graphique par ImageAjust_upd
case 'SliderEMin'	
	val = get(gcbo,'Value');
	ud = get(gcbf,'UserData');
	ud.DynMin = val;
	UpdDynamique
	set(gcbf,'UserData',ud)
case 'SliderEMax'	
	val = get(gcbo,'Value');
	ud = get(gcbf,'UserData');
	ud.DynMax = val;
	UpdDynamique
	set(gcbf,'UserData',ud)
case 'ButRot-'
	cible = IsaacImageCible;
	objIMG = IsaacImage_get(cible,'IMG');
	try
		IsaacImage_set(cible,gcbf,'IMG',IsaacIMG_rotate(objIMG,90));
	catch exception
		IsaacMessageErreur(IsaacError(exception))
	end
	% maj graphique par ImageAjust_upd
case 'ButRot+'
	cible = IsaacImageCible;
	objIMG = IsaacImage_get(cible,'IMG');
	try
		IsaacImage_set(cible,gcbf,'IMG',IsaacIMG_rotate(objIMG,270));
	catch exception
		IsaacMessageErreur(IsaacError(exception))
	end
	% maj graphique par ImageAjust_upd
case 'ButMirrorH'
	cible = IsaacImageCible;
	objIMG = IsaacImage_get(cible,'IMG');
	try
		IsaacImage_set(cible,gcbf,'IMG',IsaacIMG_flip(objIMG,'h'));
	catch exception
		IsaacMessageErreur(IsaacError(exception))
	end
	% maj graphique par ImageAjust_upd
case 'BuMirrorV'
	cible = IsaacImageCible;
	objIMG = IsaacImage_get(cible,'IMG');
	try
		IsaacImage_set(cible,gcbf,'IMG',IsaacIMG_flip(objIMG,'v'));
	catch exception
		IsaacMessageErreur(IsaacError(exception))
	end
	% maj graphique par ImageAjust_upd
end	
	
% --- Nested fonction ------------------------------------------------------------------------------

function UpdDynamique

cible = IsaacImageCible;
paramr = IsaacImage_get(cible,'ParamR');	
if (paramr.nbbits < 32)
	valmax = IsaacImage_get(cible,'ValMax');
	set(findobj(gcbf,'Tag','SliderEMin'),'Enable','on','Value',ud.DynMin)
	set(findobj(gcbf,'Tag','StxtEMinVal'),'String',num2str(round(valmax*ud.DynMin)))
	set(findobj(gcbf,'Tag','SliderEMax'),'Enable','on','Value',ud.DynMax)
	set(findobj(gcbf,'Tag','StxtEMaxVal'),'String',num2str(round(valmax*ud.DynMax)))
else
	set(findobj(gcbf,'Tag','SliderEMin'),'Enable','on','Value',ud.DynMin)
	set(findobj(gcbf,'Tag','StxtEMinVal'),'String',num2str(ud.DynMin))
	set(findobj(gcbf,'Tag','SliderEMax'),'Enable','on','Value',ud.DynMax)
	set(findobj(gcbf,'Tag','StxtEMaxVal'),'String',num2str(ud.DynMax))
end

end % fin nested fonction

%---------------------------------------------------------------------------------------------------

end % fin ImageAdjust_cb en raison des nested fonctions
