function LpixEdit()
% Editeur de @IsaacLPIX
%
% Tool for OpenIsaac
%  2018 Alain Clment - Universit d'Angers

%---------------------------------------------------------------------------------------------------

% Copyright 2007 - 2018 Universit d'Angers - Author: Alain Clment
%
% This program is free software; you can redistribute it and/or modify it under the terms of the GNU
% General Public License as published by the Free Software Foundation; either version 3 of the
% License, or any later version. This program is distributed in the hope that it will be useful, but
% WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A
% PARTICULAR PURPOSE. See the GNU General Public License for more details. You should have received
% a copy of the GNU General Public License along with this program; if not, write to the Free
% Software Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.

%---------------------------------------------------------------------------------------------------

% structure ud : champ UserData
%	ToolDir			: rpertoire de l'outil
%	PTarget			: 1 = image cible, 2 = mmoire, 3 = fichier, 4  = MATLAB base workspace
%	PVarName       : nom de la variable associe au paramtre dit
%	Param          : objet @IsaacLPIX
%	PixL           : numro de ligne slectionne dans pix_mat
%	PixE           : entre [X,Y]
%	PixSpaFormat	: format d'affichage des valeurs X et Y de pix_mat
%	PixValFormat	: format d'affichage des valeurs V de pix_mat
%	BitFormat		: format de codage des valeurs V affiches (1/8/16/64)

%---------------------------------------------------------------------------------------------------

global ISAACINFO

% si LpixEdit est dja lanc, l'ammener au premier plan.
h = findobj(allchild(0),'flat','Tag','LpixEdit');
if ~isempty(h)
	figure(h(1)) % mise au 1er plan
   return
end

hauteurfig = 325;
% figure(position selon ISAACINFO.DefPos)
h0 = figure( ...
	'CloseRequestFcn','LpixEdit_cb CloseBox', ...
	'Name','LpixEdit', ...
	'Position',IsaacFenPos(ISAACINFO.DefPos(1),ISAACINFO.DefPos(2),285,hauteurfig,0), ...
	'Tag','LpixEdit');

IsaacDefaultProperties(h0)

% text ParamInfo -----------------------------------------------------------------------------------	
uicontrol('Parent',h0, ...
	'BackgroundColor',ISAACINFO.IsaacColor2, ...
	'Enable','inactive', ...
	'FontSize',ISAACINFO.FontSize + 1, ...
	'Position',[0 hauteurfig - 15 285 15], ...
	'Style','text', ...
	'Tag','StxtParamInfo');
% liste LP 
uicontrol('Parent',h0, ...
	'BackgroundColor',ISAACINFO.IsaacColor2, ...
	'Callback','LpixEdit_cb ListLP', ...
	'Enable','off', ...
	'FontName','Monospaced', ...
	'FontSize',ISAACINFO.FontSize + 1, ...
	'SelectionHighlight','off', ...
	'Position',[5 hauteurfig - 160 200 139], ...
	'Style','listbox', ...
	'Tag','ListLP');
% text LP effectif
uicontrol('Parent',h0, ...
	'Enable','inactive', ...
	'ForegroundColor',ISAACINFO.TextColor2, ...
	'Position',[210 hauteurfig - 43 70 15], ...
	'String',' Effectif', ...
	'Style','text');
% text LP effectif val	
uicontrol('Parent',h0, ...
	'BackgroundColor',ISAACINFO.IsaacColor2, ...
	'Enable','inactive', ...
	'FontSize',ISAACINFO.FontSize + 1, ...
	'Position',[210 hauteurfig - 60 70 15], ...
	'Style','text', ...
	'Tag','StxtLPeffval');
% bouton LPselect
uicontrol('Parent',h0, ...
	'BackgroundColor',ISAACINFO.IsaacColor3, ...
	'Callback','LpixEdit_cb ButLPselect', ...
	'Enable','off', ...
	'Position',[210 hauteurfig - 84 70 20], ...
	'String','Selection', ...
	'Tag','ButLPselect');
% bouton LPadjust
uicontrol('Parent',h0, ...
	'BackgroundColor',ISAACINFO.IsaacColor3, ...
	'Callback','LpixEdit_cb ButLPadjust', ...
	'Enable','off', ...
	'Position',[210 hauteurfig - 108 70 20], ...
	'String','Ajuster', ...
	'Tag','ButLPadjust');
% bouton LPclear
uicontrol('Parent',h0, ...
	'BackgroundColor',ISAACINFO.IsaacColor3, ...
	'Callback','LpixEdit_cb ButLPclear', ...
	'Enable','off', ...
	'Position',[210 hauteurfig - 132 70 20], ...
	'String','Effacer', ...
	'Tag','ButLPclear');
% bouton LP+
uicontrol('Parent',h0, ...
	'BackgroundColor',ISAACINFO.IsaacColor3, ...
	'Callback','LpixEdit_cb ButLP+', ...
	'Enable','off', ...
	'Position',[227 hauteurfig - 156 16 16], ...
	'String','+', ...
	'Tag','ButLP+');
% bouton LP-
uicontrol('Parent',h0, ...
	'BackgroundColor',ISAACINFO.IsaacColor3, ...
	'Callback','LpixEdit_cb ButLP-', ...
	'Enable','off', ...
	'Position',[249 hauteurfig - 156 16 16], ...
	'String','-', ...
	'Tag','ButLP-');
% text legende LP	
uicontrol('Parent',h0, ...
	'Enable','inactive', ...
	'ForegroundColor',ISAACINFO.TextColor2, ...
	'Position',[10 hauteurfig - 180 80 15], ...
	'String',' X   ,   Y', ...
	'Style','text');
% edit Coord XY	
uicontrol('Parent',h0, ...
	'BackgroundColor',ISAACINFO.IsaacColor2, ...
	'Callback','LpixEdit_cb EtxtCoord', ...
	'Enable','inactive', ...
	'FontSize',ISAACINFO.FontSize, ...
	'Position',[10 hauteurfig - 200 80 22], ...
	'Style','edit', ...
	'Tag','EtxtCoord');
% text Affichage bits
uicontrol('Parent',h0, ...
	'Enable','inactive', ...
	'ForegroundColor',ISAACINFO.TextColor2, ...
	'HorizontalAlignment','left', ...
	'Position',[125 hauteurfig - 180 100 15], ...
	'String','Affichage bits :', ...
	'Style','text');
% radiobouton 1
uicontrol('Parent',h0, ...
	'Callback','LpixEdit_cb Rad1', ...
	'Position',[120 hauteurfig - 198 35 15], ...
	'String','1', ...
	'Style','radiobutton', ...
	'Tag','Rad1');
% radiobouton 8
uicontrol('Parent',h0, ...
	'Callback','LpixEdit_cb Rad8', ...
	'Position',[155 hauteurfig - 198 35 15], ...
	'String','8', ...
	'Style','radiobutton', ...
	'Tag','Rad8');
% radiobouton 16
uicontrol('Parent',h0, ...
	'Callback','LpixEdit_cb Rad16', ...
	'Position',[190 hauteurfig - 198 40 15], ...
	'String','16', ...
	'Style','radiobutton', ...
	'Tag','Rad16');
% radiobouton 64
uicontrol('Parent',h0, ...
	'Callback','LpixEdit_cb Rad64', ...
	'Position',[230 hauteurfig - 198 40 15], ...
	'String','64', ...
	'Style','radiobutton', ...
	'Tag','Rad64');
% Sparateur ---------------------------------------------------------------------------------------
uicontrol('Parent',h0, ...
	'Enable','inactive', ...
	'Position',[0 hauteurfig - 210 285 1], ...
	'Style','frame');
% text TargetInfo	
uicontrol('Parent',h0, ...
	'BackgroundColor',ISAACINFO.IsaacColor2, ...
	'Enable','inactive', ...
	'FontSize',ISAACINFO.FontSize + 1, ...
	'Position',[5 hauteurfig - 230 275 15], ...
	'Style','text', ...
	'Tag','StxtTargetInfo');
% Bouton radio Param image cible
uicontrol('Parent',h0, ...
	'Callback','LpixEdit_cb RadButParamCible', ...
	'String','Image cible', ...
	'Style','radiobutton', ...
	'Position',[5 hauteurfig - 255 190 15], ...
	'Tag','RadButParamCible');
% Bouton radio Param mmoire
uicontrol('Parent',h0, ...
	'Callback','LpixEdit_cb RadButParamMem', ...
	'String','Mmoire', ...
	'Style','radiobutton', ...
	'Position',[5 hauteurfig - 275 100 15], ...
	'Tag','RadButParamMem');
% Bouton radio Param fichier
uicontrol('Parent',h0, ...
	'Callback','LpixEdit_cb RadButParamFich', ...
	'String','Fichier', ...
	'Style','radiobutton', ...
	'Position',[5 hauteurfig - 295 100 15], ...
	'Tag','RadButParamFich');
% Bouton radio Param MATLAB
uicontrol('Parent',h0, ...
	'Callback','LpixEdit_cb RadButParamMATLAB', ...
	'String','MATLAB', ...
	'Style','radiobutton', ...
	'Position',[5 hauteurfig - 315 100 15], ...
	'Tag','RadButParamMATLAB');
% edit Varname	
uicontrol('Parent',h0, ...
	'BackgroundColor',ISAACINFO.IsaacColor2, ...
	'Callback','LpixEdit_cb EtxtVarname', ...
	'FontSize',ISAACINFO.FontSize, ...
	'Position',[90 hauteurfig - 318 80 22], ...
	'Style','edit', ...
	'Tag','EtxtVarname');
% bouton ParamPull
uicontrol('Parent',h0, ...
	'BackgroundColor',ISAACINFO.IsaacColor3, ...
	'Callback','LpixEdit_cb ButParamPull', ...
	'Position',[190 hauteurfig - 290 90 20], ...
	'Tag','ButParamPull');
% bouton ParamPush
uicontrol('Parent',h0, ...
	'BackgroundColor',ISAACINFO.IsaacColor3, ...
	'Callback','LpixEdit_cb ButParamPush', ...
	'Position',[190 hauteurfig - 315 90 20], ...
	'Tag','ButParamPush');

%---------------------------------------------------------------------------------------------------

% Initialisation

ud.ToolDir = fileparts(mfilename('fullpath'));
ud.PTarget = 1; % image cible
ud.PVarName = 'x';
ud.Param = IsaacLPIX();
ud.PixSpaFormat = '%5.4i';
ud.BitFormat = 64;
ud.PixValFormat = '%7.4f';
set(h0,'UserData',ud);
set(findobj(h0,'Tag','EtxtVarname'),'String',ud.PVarName,'Visible','off')
set(findobj(h0,'Tag','RadButParamCible'),'Value',1)
set(findobj(h0,'Tag','ButParamPull'),'Visible','off')
set(findobj(h0,'Tag','ButParamPush'),'Visible','off')
set(findobj(h0,'Tag','StxtLPeffval'),'String','0')
set(findobj(h0,'Tag','Rad64'),'Value',1)
set(findobj(h0,'Tag','StxtParamInfo'),'String','LPIX vide')
addpath(ud.ToolDir)
IsaacEventUpdListe(h0,ud.ToolDir,'LpixEdit_upd') % abonnement aux vnements
cible = IsaacImageCible;
if isempty(cible)
	LpixEdit_upd(h0,1,cible,NaN)
else
	LpixEdit_upd(h0,2,cible,NaN)
end
% auteur = NaN car n'accepte pas les vnements causs par soi-mme pour mise  jour graphique