function IsaacEngine(varargin)
% Moteur Isaac
%
% OpenIsaac
%  2018 Alain Clment - Universit d'Angers

%---------------------------------------------------------------------------------------------------

% Copyright 2007 - 2018 Universit d'Angers Author: Alain Clment 
% Contributor: Clment Bourgeais (2018)
%
% This program is free software; you can redistribute it and/or modify it under the terms of the GNU
% General Public License as published by the Free Software Foundation; either version 3 of the
% License, or any later version. This program is distributed in the hope that it will be useful, but
% WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A
% PARTICULAR PURPOSE. See the GNU General Public License for more details. You should have received
% a copy of the GNU General Public License along with this program; if not, write to the Free
% Software Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.

%---------------------------------------------------------------------------------------------------
		
% Dclaration des globales
global ISAACPRIV
global ISAACCONST

% Isaac est dja lanc ?
IsaacRunning = (exist('ISAACPRIV','var') && ~isempty(ISAACPRIV));

% traitement des appels
switch nargin
case 1
   if IsaacRunning
      switch varargin{1}
      case '-end'
         % terminaison sans condition
         info = ISAACCONST.Info;
         IsaacEnd(info,true)
      case '-endconfirm'
         % terminaison avec confirmation
         if strcmp(IsaacDialogQuestion('Confirmation quitter Isaac?',1),'Oui')
            info = ISAACCONST.Info;
            IsaacEnd(info,true)
         end
      case '-help'
         % aide html
         web (['http://',ISAACCONST.Info.IsaacDoc1],'-browser')
      case '-restart'
         % terminaison et redmarrage en mode graphique
         info = ISAACCONST.Info;
         IsaacEnd(info,false)
         IsaacStart(info,false)
         IsaacMenuBar
      case '-start'
         % dmarrage en mode graphique
         % si barre de menu lance alors l'ammener au premier plan
         h = findobj(allchild(0),'flat', 'Tag', 'IsaacMenuBar');
         if ~isempty(h)
            figure(h(1)) % mise au 1er plan
         end
         % Bannire de session Isaac en cours
         IsaacShowRun(IsaacIdentity)
      case '-lang'
         % dmarrage en mode langage seul
         % Bannire de session Isaac en cours
         IsaacShowRun(IsaacIdentity)
      otherwise
         IsaacShowError1(IsaacIdentity)
      end   
   else
      switch varargin{1}
      case '-start'
         % dmarrage en mode graphique
         IsaacStart(IsaacIdentity,true)
         IsaacMenuBar
      case '-lang'
         % dmarrage en mode langage seul
         IsaacStart(IsaacIdentity,true)
      otherwise
         IsaacShowError1(IsaacIdentity)
      end   
   end
otherwise
	IsaacShowError2(IsaacIdentity)
end

%---------------------------------------------------------------------------------------------------

function IsaacStart(ParamInfo,showstart)
	
global ISAACPREF
global ISAACPRIV
global ISAACCONST
global ISAACINFO
	
if showstart
   % Bannire de lancement Isaac
   IsaacShowStart(ParamInfo)
end   

% Configuration (objet graphique Root = 0)
format compact % Matlab command Window
set(0,'DefaultFigurePaperType','a4letter')
%set(0,'DefaultFigureRenderer','OpenGL') pour future gestion de renderer = OpenGL
set(0,'Units','pixels')

% Test crans et rsolution (cran de rfrence = cran principal) 

mp = get(0,'MonitorPosition');         
nbecrans = size(mp,1);
sc = get(0,'ScreenSize');

% BUG MATLAB 301358 
% Incorrect values for MonitorPosition and the ScreenSize on dual monitor systems
% de R14 (7.0)  2014a (8.3)

% MATLAB R14 et R2018a : si 2 crans, valeurs diffrentes de MonitorPosition et de ScreenSize

% MATLAB R14    : si l'cran 2aire est positionn plus haut que l'cran primaire
%                les positions des fentres sont fausses (0 dcal vers le bas)
% MATLAB R2018a : ce n'est plus le cas

% MATLAB R14    : MonitorPosition dduit la hauteur de la barre de menu du Mac (22 pixels)
% MATLAB R2018a : ce n'est plus le cas

% MATLAB R14    : impossible de dimensionner une fentre  moins de (L=128,H=16) pixels
% MATLAB R2018a : ce n'est plus le cas mais largeur mini = celle des boutons de la barre de titre

if verLessThan('matlab', '8.4')
	% code pour MATLAB 2014a et antrieur
   if nbecrans == 1
      ISAACCONST.Screen.refpos = [0,mp(4)-1]; % left,top en coordonnes matlab
      ISAACCONST.Screen.limits = [0,sc(3)-1,0,mp(4)-1]; % left,right,top,down /refpos
   else
      if (mp(1,2) + mp(1,4)) == sc(4) % cran 2aire positionn plus haut que l'cran primaire
         errstr = ['Incorrect values for MonitorPosition and ScreenSize\n', ...
         'BUG MATLAB 301358 from R14 (7.0) to 2014a (8.3)\n', ...
         'The top of the second screen must not exceed the primary screen\n\n'];
         fprintf(['\n',ParamInfo.IsaacName,' : ',errstr]);
      end
      ISAACCONST.Screen.refpos = [0,sc(4)-mp(1,2)-1]; % left,top en coordonnes matlab     
      ISAACCONST.Screen.limits = [1-mp(1,1),sc(3)-mp(1,1),-mp(1,2),sc(4)-mp(1,2)-1];
      % left,right,top,down /refpos     
   end
else
   % code pour MATLAB 2014b (8.4) et postrieur
   if ismac
      offset = 22; % hauteur de la barre de menu
   else
      offset = 0;
   end
   ISAACCONST.Screen.refpos = [0,sc(4)-1-offset]; % left,top en coordonnes matlab
	ISAACCONST.Screen.limits = [0,sc(3)-1,0,sc(4)-1-offset]; % left,right,top,down /refpos
	% left,right,top,down /refpos
end

ISAACCONST.MinFenSize		= [128,16]; % taille min d'une fentre avec MATLAB R14
% Dans Matlab, la position des fentres ne tient pas compte des bords
ISAACCONST.EpFen           = [20,0,0,0]; % paisseur barre du haut, du bas, de gauche, de droite
%ISAACCONST.EpFenInside		= [0,20,0,20]; pour future gestion de OpenGL (carr de resize)
ISAACCONST.EpFenMenuBar		= 28; % hauteur de la barre de menu dans une fentre
ISAACCONST.Screen.width		= 1+ISAACCONST.Screen.limits(2)-ISAACCONST.Screen.limits(1);
ISAACCONST.Screen.height	= 1+ISAACCONST.Screen.limits(4)-ISAACCONST.Screen.limits(3);
% Coin extrieur sup-gauche par dfaut pour l'ouverture des fentres
ISAACCONST.DefPos          = [50,100];
% Hauteur (en pixels) du bouton cible dans les images
ISAACCONST.EpButCible		= 7;

% rpertoire Program
ISAACCONST.ProgDir      = fileparts(which('IsaacEngine'));
% rpertoire Isaac
ISAACCONST.IsaacDir     = fileparts(ISAACCONST.ProgDir);
% rpertoire Preferences
ISAACCONST.PrefDir      = fullfile(ISAACCONST.IsaacDir,'Preferences');
% Autres rpertoires et chemins
ISAACCONST.ResDir       = fullfile(ISAACCONST.IsaacDir,'Ressources'); 
ISAACCONST.ProfilsDir   = fullfile(ISAACCONST.IsaacDir,'Profils'); 
ISAACCONST.TempDir		= fullfile(ISAACCONST.IsaacDir,'Temp');
ISAACCONST.ToolsDir		= fullfile(ISAACCONST.IsaacDir,'Outils');
ISAACCONST.ModsDir		= fullfile(ISAACCONST.IsaacDir,'Modules');

% Dfinition des autres constantes
ISAACCONST.CallbackInterrupt   = false; % pour IsaacDefaultProperties.m
ISAACCONST.Info            = ParamInfo;
ISAACCONST.MaxName         = namelengthmax;
ISAACCONST.ImMaxZ          = 99; % nb plans maximum = sur 2 chiffres
ISAACCONST.PalStr          = {'gray';'bone';'hot';'copper';'pink';'cool'; 'spring';'summer'; ...
                           'autumn';'winter';'jet';'hsv'};
ISAACCONST.ExtImg = '.isa'; % Extension format d'image Isaac

% Initialisations
ISAACPRIV.Bavard           = false;
ISAACPRIV.DispEvent			= false;
ISAACPRIV.IsEnd				= false;
ISAACPRIV.NbMod				= 0;
ISAACPRIV.NbOpenImg			= 0;
ISAACPRIV.NbUpd				= 0;
ISAACPRIV.NewImgIdx			= 0;
ISAACPRIV.ParamMem			= struct;
% pas ISAACPRIV.TabIsaacImage = [] car contient des @IsaacImage et non des double

% Lecture des prfrences ou criture des prfrences par dfaut
fname = [ParamInfo.IsaacName,ParamInfo.IsaacVersion,'Prefs.mat'];
if exist(fullfile(ISAACCONST.PrefDir,fname),'file')
	load(fullfile(ISAACCONST.PrefDir,fname),'ISAACPREF')
else
	% Prfrences par dfaut
	% rpertoire par dfaut = rpertoire parent de IsaacDir
	ISAACPREF.DefaultDir          = fileparts(ISAACCONST.IsaacDir);
	ISAACPREF.UserModsDir      	= '';
	ISAACPREF.UserToolsDir     	= '';
	ISAACPREF.Mode             	= 1;
	ISAACPREF.OptimiseSerMem      = true;
	ISAACPREF.OptimiseImgMem      = false;
	ISAACPREF.ImForm              = ISAACCONST.ExtImg;
	ISAACPREF.ImMaxPlans          = 10;
	ISAACPREF.ImFullName          = false;
	ISAACPREF.ShowImType          = true;
   ISAACPREF.Son              	= true;
   ISAACPREF.CloseMenuEnd        = true;
   ISAACPREF.FontName            = 'Arial';
	ISAACPREF.FontSize            = 9;
	ISAACPREF.IsaacColor1         = [0.5 0.6 0.7];	% bleu ardoise
	ISAACPREF.IsaacColor2      	= [0.7 0.7 0.7];	% gris clair
	ISAACPREF.IsaacColor3         = [0.9 0.9 0.9];	% gris trs clair
	ISAACPREF.TextColor1          = [0 0 0];			% noir
	ISAACPREF.TextColor2          = [1 1 1];			% blanc
	ISAACPREF.TextColor3          = [0.9 0.1 0.1];	% rouge
	ISAACPREF.CibleColor1         = [0 1 0];			% vert vif
	ISAACPREF.CibleColor2         = [1 0.5 0];		% orange vif
	ISAACPREF.ROIColor1           = im2uint8([0.1 0.1 0.8]); % bleu fonc
	ISAACPREF.ROIColor2           = im2uint8([1.0 0.8 0.1]); % jaune
	ISAACPREF.ROIColor3           = im2uint8([0.9 0.1 0.1]); % rouge
	ISAACPREF.ROIColor4           = im2uint8([0.2 0.6 0.1]); % vert
	save(fullfile(ISAACCONST.PrefDir,fname),'ISAACPREF')
end

% Recopie ISAACINFO
ISAACINFO.DefPos           = ISAACCONST.DefPos;
ISAACINFO.FontName			= ISAACPREF.FontName;
ISAACINFO.FontSize			= ISAACPREF.FontSize;
ISAACINFO.ImMaxPlans       = ISAACPREF.ImMaxPlans;
ISAACINFO.IsaacColor1		= ISAACPREF.IsaacColor1;
ISAACINFO.IsaacColor2		= ISAACPREF.IsaacColor2;
ISAACINFO.IsaacColor3		= ISAACPREF.IsaacColor3;
ISAACINFO.IsEnd				= ISAACPRIV.IsEnd;
ISAACINFO.PalStr           = ISAACCONST.PalStr;
ISAACINFO.PrefDir          = ISAACCONST.PrefDir;
ISAACINFO.ProfilsDir       = ISAACCONST.ProfilsDir;
ISAACINFO.ROIColor1			= ISAACPREF.ROIColor1;
ISAACINFO.ROIColor2			= ISAACPREF.ROIColor2;
ISAACINFO.ROIColor3			= ISAACPREF.ROIColor3;
ISAACINFO.ROIColor4			= ISAACPREF.ROIColor4;
ISAACINFO.TextColor1       = ISAACPREF.TextColor1;
ISAACINFO.TextColor2       = ISAACPREF.TextColor2;
ISAACINFO.TextColor3       = ISAACPREF.TextColor3;

% Tableau des modules 
% (en cas de doublon, priorit aux modules d'ISAACCONST.ModsDir sur ceux d'ISAACPREF.UserModsDir)
try
	IsaacModuleReload;
catch exception
	IsaacMessageErreur(IsaacError(exception))
end

% Effacement du rpertoire temporaire
try
	oldstate = recycle;
	recycle('off'); % effacement immdiat
	delete(fullfile(ISAACCONST.TempDir,'*'))
	recycle(oldstate)
catch exception
	IsaacMessageErreur(IsaacError(exception))
end	

% Rpertoire par dfaut
IsaacCD(ISAACPREF.DefaultDir)

%---------------------------------------------------------------------------------------------------

function IsaacEnd(ParamInfo,showend)
% Terminaison Isaac
	
global ISAACPRIV
global ISAACINFO
global ISAACCONST

ISAACPRIV.IsEnd = true;
% Recopie ISAACINFO
ISAACINFO.IsEnd = ISAACPRIV.IsEnd;

if showend
	% Bannire de terminaison Isaac
	IsaacShowEnd(ParamInfo)
end

% Effacement du rpertoire temporaire
try
	oldstate = recycle;
	recycle('off'); % effacement immdiat
	delete(fullfile(ISAACCONST.TempDir,'*'))
	recycle(oldstate)
catch exception
	IsaacMessageErreur(IsaacError(exception))
end
cd(ISAACCONST.IsaacDir)

% Nettoyage base workspace et fermeture fentres
closereq % default figure close request function
close all hidden 
% ##### sinon erreur : pas fermeture des fenetres ouvertes

if verLessThan('matlab', '7.6') 
	% code pour MATLAB 2007b et antrieur
	clear all                                                                                    %#ok
   % warning('off') % warning('on')
   % Dsactivation des warnings le temps du clear all
   % sinon WARNING fonction en cours d'execution non efface depuis aprs MATLAB R2010b
   % => inutile si MATLAB 2007b et antrieur
else
	% code pour MATLAB 2008a (7.6) et postrieur
	clearvars % efface le current workspace (isaacEngine)
	clearvars -global ISAACPRIV
	clearvars -global ISAACPREF
	clearvars -global ISAACCONST
	clearvars -global ISAACINFO
end
% ##### sinon erreur : en cas de relancemnt d'isaac 
% les variables globales de la session prcdente ne seront pas effaces

%---------------------------------------------------------------------------------------------------

function IsaacShowStart(structinfo)
% Bannire de lancement Isaac
fprintf(['\n',structinfo.IsaacName,' ',structinfo.IsaacVersion,'\n']);
fprintf([structinfo.Copyright,'\n\n']);

function IsaacShowRun(structinfo)
% Bannire de session Isaac en cours
fprintf(['\nsession ',structinfo.IsaacName,' en cours\n\n']);

function IsaacShowEnd(structinfo)
% Bannire de terminaison Isaac
fprintf(['\nsession ',structinfo.IsaacName,' termine\n\n']);

function IsaacShowError1(structinfo)
% Commande inconnue
fprintf(['\n',structinfo.IsaacName,' : commande inconnue\n\n']);

function IsaacShowError2(structinfo)
% Nombre incorrect de paramtres
fprintf(['\n',structinfo.IsaacName,' : nombre incorrect de paramtres\n\n']);
